package envoy.client.ui.listcell;

import java.time.format.DateTimeFormatter;
import java.util.Map;

import javafx.geometry.Insets;
import javafx.scene.control.*;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.VBox;
import javafx.stage.PopupWindow.AnchorLocation;

import envoy.client.ui.IconUtil;
import envoy.data.Message;
import envoy.data.Message.MessageStatus;
import envoy.data.User;

/**
 * Displays a single message inside the message list.
 * <p>
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>MessageListCell.java</strong><br>
 * Created: <strong>28.03.2020</strong><br>
 *
 * @author Kai S. K. Engelbart
 * @since Envoy Client v0.1-beta
 */
public class MessageListCell extends ListCell<Message> {

	private final ListView<Message> listView;

	private static User								client;
	private static final DateTimeFormatter			dateFormat		= DateTimeFormatter.ofPattern("dd.MM.yyyy HH:mm");
	private static final Map<MessageStatus, Image>	statusImages	= IconUtil.loadByEnum(MessageStatus.class, 16);

	/**
	 * @param listView the list view inside which this cell is contained
	 * @since Envoy Client v0.1-beta
	 */
	public MessageListCell(ListView<Message> listView) { this.listView = listView; }

	/**
	 * Displays the text, the data of creation and the status of a message.
	 *
	 * @since Envoy v0.1-beta
	 */
	@Override
	protected void updateItem(Message message, boolean empty) {
		super.updateItem(message, empty);
		if (empty || message == null) {
			setText(null);
			setGraphic(null);
		} else {
			// Creating the underlying VBox, the dateLabel and the textLabel
			final var	cell		= new VBox(new Label(dateFormat.format(message.getCreationDate())));
			final var	textLabel	= new Label(message.getText());
			textLabel.setWrapText(true);
			cell.getChildren().add(textLabel);
			// Setting the message status icon and background color
			if (message.getRecipientID() != client.getID()) {
				final var statusIcon = new Label("", new ImageView(statusImages.get(message.getStatus())));
				statusIcon.setPadding(new Insets(1, 0, 5, 5));
				cell.getChildren().add(statusIcon);
				cell.getStyleClass().add("own-message");
			} else cell.getStyleClass().add("received-message");
			// Adjusting height and weight of the cell to the corresponding ListView
			cell.paddingProperty().setValue(new Insets(5, 20, 5, 20));
			if (listView != null) cell.prefWidthProperty().bind(listView.widthProperty().subtract(40));
			// Creating the Tooltip to deselect a message
			final var tooltip = new Tooltip("You can select a message by clicking on it \nand deselect it by pressing \"ctrl\" and clicking on it");
			tooltip.setWrapText(true);
			tooltip.setAnchorLocation(AnchorLocation.WINDOW_TOP_LEFT);
			setTooltip(tooltip);
			setGraphic(cell);
		}
	}

	/**
	 * @param client the user who chats with another person
	 * @since Envoy Client v0.1-beta
	 */
	public static void setUser(User client) { MessageListCell.client = client; }
}
